@@ -164,7 +164,7 @@ class UsersRemoteManager(VkontakteManager):
 #
 #        return instances
 
-    def fetch_instance_likes(self, instance, likes_type, owner_id, item_id, offset=0, count=1000, filter='likes', *args, **kwargs):
+    def fetch_likes_user_ids(self, likes_type, owner_id, item_id, offset=0, count=1000, filter='likes', *args, **kwargs):
         if count > 1000:
             raise ValueError("Parameter 'count' can not be more than 1000")
         if filter not in ['likes', 'copies']:
@@ -205,18 +205,32 @@ class UsersRemoteManager(VkontakteManager):
         # Максимальное значение параметра 1000, если не задан параметр friends_only, в противном случае 100.
         kwargs['count'] = int(count)
 
-        m2m_field_name = kwargs.pop('m2m_field_name', 'like_users')
-        like_users = getattr(instance, m2m_field_name)
-
         response = api_call('likes.getList', **kwargs)
-        ids = response['users']
+        return response['users']
 
-        # fetch users
-        users = self.fetch(ids=ids, only_expired=True) if ids else self.none()
-        for user in users:
-            like_users.add(user)
+    def fetch_instance_likes(self, instance, *args, **kwargs):
 
-        return users
+        m2m_field_name = kwargs.pop('m2m_field_name', 'like_users')
+        m2m_model = getattr(instance, m2m_field_name).through
+
+        ids = self.fetch_likes_user_ids(*args, **kwargs)
+
+        if ids:
+            # fetch users
+            users = self.fetch(ids=ids, only_expired=True).values_list('pk', flat=True)
+
+            try:
+                rel_field_name = [field.name for field in m2m_model._meta.local_fields if field.rel and field.rel.to == instance.__class__][0]
+            except IndexError:
+                raise ImproperlyConfigured("Impossible to find name of relation attribute for instance %s in m2m like users table" % instance)
+
+            # delete old relations and make new
+            m2m_model.objects.filter(**{'user_id__in': users, rel_field_name: instance}).delete()
+            m2m_model.objects.bulk_create([m2m_model(**{'user_id': user_pk, rel_field_name: instance}) for user_pk in users])
+
+            return users
+        else:
+            return self.none()
 
 
 class UserRelative(models.Model):
